@@ -1,20 +1,20 @@
 # Copyrights 2006-2008 by Mark Overmeer.
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 1.03.
+# Pod stripped from pm file by OODoc 1.04.
 use warnings;
 use strict;
 
 package XML::Compile::Schema::BuiltInFacets;
 use vars '$VERSION';
-$VERSION = '0.69';
+$VERSION = '0.70';
 use base 'Exporter';
 
-use Log::Report 'xml-compile', syntax => 'SHORT';
 
 our @EXPORT = qw/builtin_facet/;
 
-use Math::BigInt;
+use Log::Report     'xml-compile', syntax => 'SHORT';
+use Math::BigInt    try => 'GMP';
 use Math::BigFloat;
 
 use constant DBL_MAX_DIG => 15;
