@@ -1,16 +1,17 @@
 # Copyrights 2006-2008 by Mark Overmeer.
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 1.03.
+# Pod stripped from pm file by OODoc 1.04.
 use warnings;
 use strict;
 
 package XML::Compile::Schema::Translate;
 use vars '$VERSION';
-$VERSION = '0.69';
+$VERSION = '0.70';
 
 # Errors are either in class 'usage': called with request
 #                         or 'schema': syntax error in schema
+
 use Log::Report 'xml-compile', syntax => 'SHORT';
 use List::Util  'first';
 
@@ -553,8 +554,9 @@ sub particle($)
     $max = 'unbounded'
         if $max ne 'unbounded' && $max > 1 && !$self->{check_occurs};
 
-    return ()
-        if $max ne 'unbounded' && $max==0;
+#??
+#   return ()
+#       if $max ne 'unbounded' && $max==0;
 
     $min = 0
         if $max eq 'unbounded' && !$self->{check_occurs};
@@ -572,8 +574,7 @@ sub particle($)
     defined $label
         or return ();
 
-    return ($label =>
-     $self->make(block_handler => $where, $label, $min, $max, $process, $local))
+    return $self->make(block_handler => $where, $label, $min, $max, $process, $local)
         if ref $process eq 'BLOCK';
 
     my $required = $min==0 ? undef
@@ -708,7 +709,7 @@ sub particleElement($)
     my $nillable = $node->getAttribute('nillable') || 'false';
     $self->assertType($where, nillable => boolean => $nillable);
 
-    my $do       = $self->element($tree->descend($node));
+    my $do       = $self->element($tree->descend($node, $name));
 
     my $generate
      = $self->isTrue($nillable) ? 'element_nillable'
@@ -761,7 +762,7 @@ sub attributeOne($)
             or error __x"ref attribute without name at {where}"
                  , where => $tree->path, class => 'schema';
 
-        if(my $typeattr = $ref->getAttribute('type'))
+        if($typeattr = $ref->getAttribute('type'))
         {   # postpone interpretation
         }
         else
@@ -797,16 +798,14 @@ sub attributeOne($)
         $form       = $node->getAttribute('form');
     }
 
-    my $where = $tree->path.'@'.$name;
+    my $where = $tree->path.'/@'.$name;
     $self->assertType($where, name => NCName => $name);
     $self->assertType($where, type => QName => $typeattr)
         if $typeattr;
 
-    my $path    = $tree->path . "/at($name)";
-
     unless($type)
     {   my $typename = defined $typeattr
-          ? $self->rel2abs($path, $node, $typeattr)
+          ? $self->rel2abs($where, $node, $typeattr)
           : $self->anyType($node);
 
          $type  = $self->typeByName($tree, $typename);
@@ -824,7 +823,7 @@ sub attributeOne($)
             , form => $form, where => $where, class => 'schema';
 
     my $trans   = $qual ? 'tag_qualified' : 'tag_unqualified';
-    my $tag     = $self->make($trans => $path, $node, $name);
+    my $tag     = $self->make($trans => $where, $node, $name);
     my $ns      = $qual ? $self->{tns} : '';
 
     my $use     = $node->getAttribute('use') || '';
@@ -844,7 +843,7 @@ sub attributeOne($)
      :                       'attribute';
 
     my $value = defined $default ? $default : $fixed;
-    my $do    = $self->make($generate => $path, $ns, $tag, $st, $value);
+    my $do    = $self->make($generate => $where, $ns, $tag, $st, $value);
     defined $do ? ($name => $do) : ();
 }
 
@@ -1066,7 +1065,7 @@ sub simpleContent($)
     return $self->simpleContentRestriction($tree->descend)
         if $name eq 'restriction';
 
-     error __x"simpleContent either extension or restriction, not `{name}' at {where}"
+     error __x"simpleContent needs extension or restriction, not `{name}' at {where}"
          , name => $name, where => $tree->path, class => 'schema';
 }
 
@@ -1169,7 +1168,7 @@ sub complexContent($)
     return $self->complexBody($tree->descend)
         if $name eq 'restriction';
 
-    error __x"complexContent expects either an extension or restriction, not `{name}' at {where}"
+    error __x"complexContent needs extension or restriction, not `{name}' at {where}"
         , name => $name, where => $tree->path, class => 'schema';
 }
 
