@@ -1,10 +1,10 @@
 # Copyrights 2006-2008 by Mark Overmeer.
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 1.03.
+# Pod stripped from pm file by OODoc 1.04.
 package XML::Compile::Schema::XmlReader;
 use vars '$VERSION';
-$VERSION = '0.69';
+$VERSION = '0.70';
 
 use strict;
 use warnings;
@@ -156,8 +156,8 @@ sub block_handler
     # flatten the HASH: when a block appears only once, there will
     # not be an additional nesting in the output tree.
     if($max ne 'unbounded' && $max==1)
-    {   return $process if $min==1;
-        return bless     # $min==0
+    {   return ($label => $process) if $min==1;
+        my $code =      # $min==0
         sub { my $tree    = shift or return ();
               my $starter = $tree->currentChild or return;
               my @pairs   = try { $process->($tree) };
@@ -170,11 +170,12 @@ sub block_handler
               elsif($@) {$@->reportAll};
 
               @pairs;
-            }, 'BLOCK';
+            };
+         return ($label => bless($code, 'BLOCK'));
     }
 
     if($max ne 'unbounded' && $min>=$max)
-    {   return bless
+    {   my $code =
         sub { my $tree = shift;
               my @res;
               while(@res < $min)
@@ -182,11 +183,12 @@ sub block_handler
                   push @res, {@pairs};
               }
               ($multi => \@res);
-            }, 'BLOCK';
+            };
+         return ($label => bless($code, 'BLOCK'));
     }
 
     if($min==0)
-    {   return bless
+    {   my $code =
         sub { my $tree = shift or return ();
               my @res;
               while($max eq 'unbounded' || @res < $max)
@@ -205,10 +207,11 @@ sub block_handler
               }
 
               @res ? ($multi => \@res) : ();
-            }, 'BLOCK';
+            };
+         return ($label => bless($code, 'BLOCK'));
     }
 
-    bless
+    my $code =
     sub { my $tree = shift or error __xn
              "block with `{name}' is required at least once at {path}"
            , "block with `{name}' is required at least {_count} times at {path}"
@@ -234,11 +237,14 @@ sub block_handler
               push @res, {@pairs};
           }
           ($multi => \@res);
-        }, 'BLOCK';
+        };
+
+    ($label => bless($code, 'BLOCK'));
 }
 
 sub element_handler
 {   my ($path, $args, $label, $min, $max, $required, $optional) = @_;
+    $max eq "0" and return sub {};
 
     if($max ne 'unbounded' && $max==1)
     {   return $min==1
